/***********************************************************************/
/*  The generation of a 123 worksheet from C                           */
/*  Garry J. Vass  [72307,3311]                                        */
/*  June 30, 1988                                                      */
/***********************************************************************/
/*  This program will compile without errors in Turbo C 1.5 and MSC    */
/*  5.1 (large model in both cases).  Assumptions regarding byte       */
/*  ordering obviate portability beyond the PC environment.            */
/*                                                                     */
/***********************************************************************/
#include <fcntl.h>
#include <io.h>
#include <stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/***********************************************************************/
/*                                                                     */
/*  Program function prototypes                                        */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
extern  void load_double(double value,unsigned int column,unsigned int row);
extern  void load_integer(int value,unsigned int column,unsigned int row);
extern  void load_label(unsigned char *label,unsigned int column,unsigned int row);
extern  void load_random(unsigned int column,unsigned int row);
extern  void main(void );
extern  void wk_write(int handle,void *buf,unsigned int nbyte);
/***********************************************************************/
/*  Manifest constants.  Note that the maximum size of a 123 label is  */
/*  240 characters.  This program has been set up to create a worksheet*/
/*  called "erase_me.wk1".                                             */
/*                                                                     */
/***********************************************************************/
#define MAX_123_LABEL_SIZE  240
#define WK1_FILE_NAME       "ERASE_ME.WK1"
/***********************************************************************/
/* Global variables.  Note that the 123 bof and eof templates are set  */
/* for LOTUS 2/+.  They are essentially constants.                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
signed   int  ioresult;
signed   int  wkhandle;
unsigned char ieee[8];
unsigned char wbof[6]   = {0,0,2,0,4,4};
unsigned char weof[4]   = {1,0,0,0};
unsigned char zero = 0;
/***********************************************************************/
/* 123 template for the header portion of a label.  Using this design, */
/* the length, column, and row values must be inserted on each pass.   */
/* This template must be followed by an ASCIIZ string.  The value for  */
/* length is strlen(target) + 7.  Remember that 123 rows and columns   */
/* start with zero.                                                    */
/***********************************************************************/
unsigned char template_label_header[10]  =
                {
				0x0f, 0x00,  /* opcode */
                0x00, 0x00,  /* length */
  				0xff,        /* format */
				0x00, 0x00,  /* column */
				0x00, 0x00,  /* row    */
                '\''         /* left alignment */
				};
/***********************************************************************/
/*  123 template for a double precision value (actually the only       */
/*  floating point values).  The column and row values must be inserted*/
/*  on each pass.  This template should be followed by the 8 byte      */
/*  ieee representation of the floating point value.                   */
/***********************************************************************/
unsigned char template_double[9] =
                  {
				  0x0e, 0x00,  /* opcode  */
				  0x0d, 0x00,  /* length  */
				  0xff,        /* format  */
				  0x00, 0x00,  /* column  */
				  0x00, 0x00   /* row     */
				  };
/***********************************************************************/
/* 123 template for an integer.  Columns and rows must be "plugged"    */
/* as above.  The last word of this template should contain the        */
/* value of the integer.                                               */
/*                                                                     */
/***********************************************************************/
unsigned char template_integer[11] =
                  {
				  0x0d, 0x00,  /* opcode  */
				  0x07, 0x00,  /* length  */
				  0xff,        /* format  */
				  0x00, 0x00,  /* column  */
				  0x00, 0x00,  /* row     */
				  0x00, 0x00   /* value   */
				  };
/***********************************************************************/
/* 123 template for the @RAND function.  Columns and rows must be      */
/* "plugged" as described above.                                       */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
unsigned char template_random[] =
			      {
				  0x10, 0x00,  /* opcode */
				  0x11, 0x00,  /* length */
				  0xff,        /* format */
				  0x00, 0x00,  /* column */
				  0x00, 0x00,  /* row    */
				  0x00, 0x00, 0x00,  /* @RAND  */
				  0x9b, 0x46, 0x81,
				  0xb9, 0x3f, 0x02,
				  0x00, 0x35, 0x03
				  };
/***********************************************************************/
/* Generalized service function for performing i/o.                    */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void wk_write
	(
	int      handle,
	void     *buf,
	unsigned nbyte
	)
{
ioresult = write(handle, buf, nbyte);
if (ioresult <=0)
	{
	printf("Write error\n");
	exit(0);
	}
}
/***********************************************************************/
/* Function for inserting the @RAND function into a worksheet at       */
/* the specified coordinates.                                          */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void load_random
	(
	unsigned int column,
	unsigned int row
	)
{
memmove(&template_random[5], &column, 2);
memmove(&template_random[7], &row,    2);
wk_write(wkhandle,  template_random, sizeof(template_random));
}
/***********************************************************************/
/* Function for inserting a specified integer value into a worksheet   */
/* at specifed coordinates.                                            */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void load_integer
	(
	signed   int value,
	unsigned int column,
	unsigned int row
	)
{
memmove(&template_integer[5], &column, 2);
memmove(&template_integer[7], &row,    2);
memmove(&template_integer[9], &value,  2);
wk_write(wkhandle, template_integer, sizeof(template_integer));
}
/***********************************************************************/
/* Function for inserting a specified floating point value into a      */
/* worksheet at specified coordinates.                                 */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void load_double
	(
	double value,
	unsigned int column,
	unsigned int row
	)
{
memmove(ieee,        &value,  8);
memmove(&template_double[5], &column, 2);
memmove(&template_double[7], &row,    2);
wk_write(wkhandle, template_double, sizeof(template_double));
wk_write(wkhandle, ieee, 8);
}
/***********************************************************************/
/* Function for inserting a specified label into a worksheet at        */
/* specified coordinates.                                              */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void load_label
	(
	unsigned char *label,
	unsigned int column,
	unsigned int row
	)
{
if (strlen(label) > MAX_123_LABEL_SIZE)
	{
	printf("Label is too large\n");
	exit(0);
	}
template_label_header[2] = strlen(label) + 7;
memmove(&template_label_header[5], &column, 2);
memmove(&template_label_header[7], &row,    2);
wk_write(wkhandle, template_label_header, sizeof(template_label_header));
wk_write(wkhandle, label, strlen(label));
wk_write(wkhandle, &zero, 1);
}
/***********************************************************************/
/* Begin main.  This program creates a worksheet with column headers   */
/* and proceeds to fill it with strings, doubles, integers, and random */
/* numbers.                                                            */
/*                                                                     */
/***********************************************************************/
void main()
{
int i;
wkhandle = open(WK1_FILE_NAME,
                O_RDWR  | O_CREAT | O_BINARY,
				S_IREAD | S_IWRITE);
if (wkhandle < 0)
	{
	printf("Unable to get file handle\n");
	exit(0);
	}
/***********************************************************************/
/* Write the bof record.                                               */
/***********************************************************************/
wk_write(wkhandle, wbof, sizeof(wbof));
/***********************************************************************/
/*  Set up the column headers.                                         */
/***********************************************************************/
load_label("TEXT",     0, 0);
load_label("FLOATS",   2, 0);
load_label("INTEGERS", 4, 0);
load_label("RANDOMS",  6, 0);
for (i=0;i<=6;++i)
	{
	load_label("========", i, 1);
	}
/***********************************************************************/
/*  Fill out the worksheet...                                          */
/***********************************************************************/
for (i=2;i<20;++i)
	{
    load_label  ("HELLO WORLD",       0, i);
	load_double (((double)(i) * 2.5), 2, i);
	load_integer(i,                   4, i);
	load_random (                     6, i);
	}
/***********************************************************************/
/* Write the eof record.                                               */
/***********************************************************************/
wk_write(wkhandle, weof, sizeof(weof));
(void) close(wkhandle);
printf("\nProcessing complete\n");
}
/***********************************************************************/
/* End of make123.c                                                    */
/***********************************************************************/
/* The size of the worksheet file created by this program is about     */
/* 1500 bytes.  If the LOTUS /fs option is used, the resulting file    */
/* size will be about 3000 bytes, because 123 adds in overhead (for    */
/* CALC precedence, etc) when saving worksheets.                       */
/***********************************************************************/